<?php
if(!defined('IN_DISCUZ')) {
	exit('Access Denied');
}
//获取js临时token
include_once('bphp.func.php');

//https://help.aliyun.com/zh/oss/developer-reference/authorize-access-2?spm=a2c4g.11186623.0.0.3f0a22c71zHcMq#section-8rj-9sk-q7r
//获取临时访问凭证前，您需要使用composer require alibabacloud/sts-20150401命令安装STS依赖。
// source\plugin\bphp_clouds\vendor\aliyunJsToken\guzzlehttp\guzzle\src\Client.php   关闭https 'verify' => false,

require_once __DIR__ . '/vendor/aliyunJsToken/autoload.php';
use AlibabaCloud\SDK\Sts\V20150401\Sts;
use Darabonba\OpenApi\Models\Config;
use AlibabaCloud\SDK\Sts\V20150401\Models\AssumeRoleRequest;
use AlibabaCloud\Tea\Utils\Utils\RuntimeOptions;


//阿里云oss
function aliyunOssToken(){
	if(isset($_COOKIE["aliyuntoken-".$_G['uid']])){
		return ['sta'=>1,'msg'=>'获取阿里云token成功','data'=> json_decode($_COOKIE["aliyuntoken"],true)];
	}
	
	global $_G;
	$pConfig = bphpConfig();
	
	
	//权限限制
	$format = explode(',',$pConfig['format']);
	$allowExtRule = '';
	foreach($format as $k=>$v){
		$allowExtRule .="\"acs:oss:*:*:".$pConfig['aliyun']['bucket']."/bphpClouds/".$_G['uid']."/*".$v."\",";
	}
	$allowExtRule = substr($allowExtRule,0,-1);
	$policy = "{
				\"Version\": \"1\",
				\"Statement\": [
				 {
					   \"Effect\": \"Allow\",
					   \"Action\": [
						 \"oss:PutObject\",
						 \"oss:GetObject\",
						 \"oss:DeleteObject\"
					   ],
					   \"Resource\": [
						 ".$allowExtRule ."
					   ]
				 }
				]
			}";
	
	//debug($policy);

	try {
		 // 填写步骤1创建的RAM用户AccessKey。
		$config = new Config([        
			"accessKeyId" => $pConfig['aliyun']['accessKey'],
			"accessKeySecret" => $pConfig['aliyun']['accessKeySecret'],
		]);
		$config->endpoint = 'sts.'.$pConfig['aliyun']['regionId'].'.aliyuncs.com';
		$client =  new Sts($config);   

		$assumeRoleRequest = new AssumeRoleRequest([
			// roleArn填写步骤2获取的角色ARN，例如acs:ram::175708322470****:role/ramtest。        
			"roleArn" => $pConfig['aliyun']['arn'],
			// roleSessionName用于自定义角色会话名称，用来区分不同的令牌，例如填写为sessiontest。
			"roleSessionName" => $pConfig['aliyun']['ram'],
			// durationSeconds用于设置临时访问凭证有效时间单位为秒，最小值为900，最大值以当前角色设定的最大会话时间为准。本示例指定有效时间为3000秒。
			"durationSeconds" => 3000,
			// policy填写自定义权限策略，用于进一步限制STS临时访问凭证的权限。如果不指定Policy，则返回的STS临时访问凭证默认拥有指定角色的所有权限。
			// 临时访问凭证最后获得的权限是步骤4设置的角色权限和该Policy设置权限的交集。
			"policy" => $policy 
		]);
		
		$runtime = new RuntimeOptions([]);
		$result = $client->assumeRoleWithOptions($assumeRoleRequest, $runtime);
		$securityToken = $result->body->credentials->securityToken;
		$accessKeyId = $result->body->credentials->accessKeyId;
		$accessKeySecret = $result->body->credentials->accessKeySecret;
		
		$data = [
			'stsToken'=> $securityToken,
			'accessKeyId'=>$accessKeyId,
			'accessKeySecret'=>$accessKeySecret,
			'region'=>'oss-'.$pConfig['aliyun']['regionId'],
			'bucket'=>$pConfig['aliyun']['bucket'],
		];
		
		setcookie("aliyuntoken-".$_G['uid'],json_encode($data), time()+2999);
		return ['sta'=>1,'msg'=>'获取阿里云token成功','data'=> $data];
		
	}catch (Exception $e){
		return ['sta'=>0,'msg'=>'阿里云储存授权失败，请检查阿里云oss配置!','errMsg'=>$e];
	}  		
}


?>